"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Model = void 0;
const DBFilter_1 = __importDefault(require("./parsing/DBFilter"));
const DBPastprocessing_1 = __importDefault(require("./parsing/DBPastprocessing"));
const DBPreprocessing_1 = __importDefault(require("./parsing/DBPreprocessing"));
const DataValidator_1 = __importDefault(require("./validation/DataValidator"));
class Model {
    constructor(name, schema, exec = {}) {
        this._dependentRefs = [];
        this.name = name;
        this.schema = schema;
        this.exec = exec;
    }
    get core() {
        return this._core;
    }
    init(core) {
        this._core = core;
        this._dependentRefs = [];
        // Check default value
        for (const name in this.schema) {
            const data = this.schema[name];
            if (!data.require) {
                data.require = false;
            }
            if (!data.export) {
                data.export = true;
            }
        }
        // Read children
        for (const typeName in this.schema) {
            // Read type
            const type = this.schema[typeName];
            // If a ref
            if (type.type === "Refs") {
                const depend = this.core.getModel(type.model);
                depend.addRefs(this, typeName, type.idName, type.dependent);
            }
        }
    }
    addRefs(model, name, idName, depend) {
        this._dependentRefs.push({
            model,
            name,
            idName,
            depend
        });
    }
    find(filter, option) {
        return __awaiter(this, void 0, void 0, function* () {
            return this._core.dataBase.findByModel(this, DBFilter_1.default(this, filter), option).then((data) => {
                const values = [];
                for (const value of data) {
                    values.push(DBPastprocessing_1.default(this, value));
                }
                return values;
            });
        });
    }
    insert(data, protection = true) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.valid(data, true, protection).then((errors) => __awaiter(this, void 0, void 0, function* () {
                if (errors.length > 0) {
                    return errors;
                }
                return this._core.dataBase.insertByModel(DBPreprocessing_1.default(this, data, true), this).then((result) => {
                    let useResult = result;
                    if (useResult) {
                        useResult = DBPastprocessing_1.default(this, useResult);
                        this.core.trigger.triggerInsert(this, useResult);
                    }
                    return useResult;
                });
            }));
        });
    }
    delete(filter = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.find(filter).then(values => {
                this.deleteManyByRefs(values);
                return this._core.dataBase.deleteByModel(this, DBFilter_1.default(this, filter)).then(() => {
                    for (const value of values) {
                        this.core.trigger.triggerDelete(this, value);
                    }
                });
            });
        });
    }
    update(data, filter, protection = true) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.valid(data, protection).then((errors) => {
                if (errors.length === 0) {
                    const updatesData = this.find(filter);
                    return this._core.dataBase.updateByModel(DBPreprocessing_1.default(this, data), this, DBFilter_1.default(this, filter)).then(() => {
                        updatesData.then(values => {
                            for (const value of values) {
                                this.core.trigger.triggerUpdate(this, value, data);
                            }
                        });
                        return [];
                    });
                }
                return errors;
            });
        });
    }
    toExport(data, exportSchema) {
        let useSchema = exportSchema;
        if (!useSchema) {
            useSchema = this.schema;
        }
        const values = {};
        for (const key in data) {
            const schema = useSchema[key];
            if (schema && schema.export) {
                if (schema.exportName) {
                    values[schema.exportName] = data[key];
                }
                else {
                    values[key] = data[key];
                }
            }
        }
        return values;
    }
    valid(data, require = false, protection = true) {
        return __awaiter(this, void 0, void 0, function* () {
            return DataValidator_1.default(data, this, require, protection);
        });
    }
    deleteManyByRefs(values) {
        return __awaiter(this, void 0, void 0, function* () {
            const task = [];
            for (const value of values) {
                task.push(this.deleteByRefs(value));
            }
            return Promise.all(task).then();
        });
    }
    deleteByRefs(value) {
        return __awaiter(this, void 0, void 0, function* () {
            const task = [];
            for (const { model, depend, idName, name } of this._dependentRefs) {
                if (value[idName]) {
                    const filter = {
                        [name]: value[idName]
                    };
                    if (depend) {
                        task.push(model.delete(filter));
                    }
                    else {
                        task.push(model.update({ [name]: null }, filter));
                    }
                }
            }
            return Promise.all(task).then();
        });
    }
}
exports.Model = Model;
//# sourceMappingURL=Model.js.map